@@ -1,9 +1,10 @@
 //! Axis Aligned Bounding Box.
 
 use crate::bounding_volume::{BoundingSphere, BoundingVolume};
-use crate::math::{Isometry, Point, Real, UnitVector, Vector, DIM};
+use crate::math::{Isometry, Point, Real, UnitVector, Vector, DIM, TWO_DIM};
 use crate::shape::{Cuboid, SupportMap};
 use crate::utils::IsometryOps;
+use arrayvec::ArrayVec;
 use na;
 use num::Bounded;
 
@@ -12,6 +13,7 @@ use na::ComplexField; // for .abs()
 
 /// An Axis Aligned Bounding Box.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "cuda", derive(cust_core::DeviceCopy))]
 #[derive(Debug, PartialEq, Copy, Clone)]
 pub struct AABB {
     pub mins: Point<Real>,
@@ -150,6 +152,16 @@ impl AABB {
         AABB::new(center + (-ws_half_extents), center + ws_half_extents)
     }
 
+    #[inline]
+    pub fn scaled(self, scale: &Vector<Real>) -> Self {
+        let a = self.mins.coords.component_mul(&scale);
+        let b = self.maxs.coords.component_mul(&scale);
+        Self {
+            mins: a.inf(&b).into(),
+            maxs: a.sup(&b).into(),
+        }
+    }
+
     /// The smallest bounding sphere containing this AABB.
     #[inline]
     pub fn bounding_sphere(&self) -> BoundingSphere {
@@ -170,6 +182,87 @@ impl AABB {
         true
     }
 
+    /// Computes the intersection of this AABB and another one.
+    pub fn intersection(&self, other: &AABB) -> Option<AABB> {
+        let result = AABB {
+            mins: Point::from(self.mins.coords.sup(&other.mins.coords)),
+            maxs: Point::from(self.maxs.coords.inf(&other.maxs.coords)),
+        };
+
+        for i in 0..DIM {
+            if result.mins[i] > result.maxs[i] {
+                return None;
+            }
+        }
+
+        Some(result)
+    }
+
+    /// Returns the difference between this AABB and `rhs`.
+    ///
+    /// Removing another AABB from `self` will result in zero, one, or up to 4 (in 2D) or 8 (in 3D)
+    /// new smaller AABBs.
+    pub fn difference(&self, rhs: &AABB) -> ArrayVec<Self, TWO_DIM> {
+        self.difference_with_cut_sequence(rhs).0
+    }
+
+    /// Returns the difference between this AABB and `rhs`.
+    ///
+    /// Removing another AABB from `self` will result in zero, one, or up to 4 (in 2D) or 8 (in 3D)
+    /// new smaller AABBs.
+    ///
+    /// # Return
+    /// This returns a pair where the first item are the new AABBs and the the second item is
+    /// the sequance of cuts applied to `self` to obtain the new AABBs. Each cut is performed
+    /// along one axis identified by `-1, -2, -3` for `-X, -Y, -Z` and `1, 2, 3` for `+X, +Y, +Z`, and
+    /// the plane’s bias.
+    /// The cuts are applied sequancially. For example, if `result.1[0]` contains `1`, then it means
+    /// that `result.0[0]` is equal to the piece of `self` lying in the negative half-space delimited
+    /// by the plane with outward normal `+X`. Then, the other piece of `self` generated by this cut
+    /// (i.e. the piece of `self` lying in the positive half-space delimited by the plane with outward
+    /// normal `+X`) is the one that will be affected by the next cut.
+    ///
+    /// The returned cut sequence will be empty if the aabbs are disjoint.
+    pub fn difference_with_cut_sequence(
+        &self,
+        rhs: &AABB,
+    ) -> (ArrayVec<Self, TWO_DIM>, ArrayVec<(i8, Real), TWO_DIM>) {
+        let mut result = ArrayVec::new();
+        let mut cut_sequence = ArrayVec::new();
+
+        // NOTE: special case when the boxes are disjoint.
+        //       This isn’t exactly the same as `!self.intersects(rhs)`
+        //       because of the equality.
+        for i in 0..DIM {
+            if self.mins[i] >= rhs.maxs[i] || self.maxs[i] <= rhs.mins[i] {
+                result.push(*self);
+                return (result, cut_sequence);
+            }
+        }
+
+        let mut rest = *self;
+
+        for i in 0..DIM {
+            if rhs.mins[i] > rest.mins[i] {
+                let mut fragment = rest;
+                fragment.maxs[i] = rhs.mins[i];
+                rest.mins[i] = rhs.mins[i];
+                result.push(fragment);
+                cut_sequence.push((i as i8 + 1, rhs.mins[i]));
+            }
+
+            if rhs.maxs[i] < rest.maxs[i] {
+                let mut fragment = rest;
+                fragment.mins[i] = rhs.maxs[i];
+                rest.maxs[i] = rhs.maxs[i];
+                result.push(fragment);
+                cut_sequence.push((-(i as i8 + 1), -rhs.maxs[i]));
+            }
+        }
+
+        (result, cut_sequence)
+    }
+
     /// Computes the vertices of this AABB.
     #[inline]
     #[cfg(feature = "dim2")]
