@@ -11,30 +11,25 @@ impl MassProperties {
         let (ball_vol, ball_unit_i) = Self::ball_volume_unit_angular_inertia(radius);
         let cap_vol = cyl_vol + ball_vol;
         let cap_mass = cap_vol * density;
-        let mut cap_unit_i = cyl_unit_i + ball_unit_i;
+        let mut cap_i = (cyl_unit_i * cyl_vol + ball_unit_i * ball_vol) * density;
         let local_com = na::center(&a, &b);
 
         #[cfg(feature = "dim2")]
         {
             let h = half_height * 2.0;
-            let extra = h * h * 0.5 + h * radius * 3.0 / 8.0;
-            cap_unit_i += extra;
-            Self::new(local_com, cap_mass, cap_unit_i * cap_mass)
+            let extra = (h * h * 0.25 + h * radius * 3.0 / 8.0) * ball_vol * density;
+            cap_i += extra;
+            Self::new(local_com, cap_mass, cap_i)
         }
 
         #[cfg(feature = "dim3")]
         {
             let h = half_height * 2.0;
-            let extra = h * h * 0.5 + h * radius * 3.0 / 8.0;
-            cap_unit_i.x += extra;
-            cap_unit_i.z += extra;
+            let extra = (h * h * 0.25 + h * radius * 3.0 / 8.0) * ball_vol * density;
+            cap_i.x += extra;
+            cap_i.z += extra;
             let local_frame = Capsule::new(a, b, radius).rotation_wrt_y();
-            Self::with_principal_inertia_frame(
-                local_com,
-                cap_mass,
-                cap_unit_i * cap_mass,
-                local_frame,
-            )
+            Self::with_principal_inertia_frame(local_com, cap_mass, cap_i, local_frame)
         }
     }
 }
