@@ -41,6 +41,7 @@ pub use self::point::{PointProjection, PointQuery, PointQueryWithLocation};
 pub use self::query_dispatcher::PersistentQueryDispatcher;
 pub use self::query_dispatcher::{QueryDispatcher, QueryDispatcherChain};
 pub use self::ray::{Ray, RayCast, RayIntersection, SimdRay};
+pub use self::split::SplitResult;
 pub use self::time_of_impact::{time_of_impact, TOIStatus, TOI};
 
 mod clip;
@@ -60,6 +61,7 @@ pub mod point;
 mod query_dispatcher;
 mod ray;
 pub mod sat;
+mod split;
 mod time_of_impact;
 #[cfg(feature = "std")]
 pub mod visitors;
